@@ -8,6 +8,7 @@
 import re
 import time
 from urllib.parse import parse_qsl, urlsplit
+import warnings
 
 import pytest
 from flask import Flask
@@ -17,8 +18,7 @@ from tests.test_utils import (
     logout,
 )
 
-from flask_security.core import UserMixin
-from flask_security.signals import login_instructions_sent
+from flask_security import Security, UserMixin, login_instructions_sent
 
 pytestmark = pytest.mark.passwordless()
 
@@ -46,6 +46,7 @@ def test_passwordless_flag(app, client, get_message):
     )
     assert response.status_code == 200
     assert len(recorded) == 1
+    assert len(app.mail.outbox) == 1
 
     # Test login with json and invalid email
     data = dict(email="nobody@lp.com")
@@ -56,14 +57,13 @@ def test_passwordless_flag(app, client, get_message):
 
     # Test sends email and shows appropriate response
     with capture_passwordless_login_requests() as requests:
-        with app.mail.record_messages() as outbox:
-            response = client.post(
-                "/login", data=dict(email="matt@lp.com"), follow_redirects=True
-            )
+        response = client.post(
+            "/login", data=dict(email="matt@lp.com"), follow_redirects=True
+        )
 
     assert len(recorded) == 2
     assert len(requests) == 1
-    assert len(outbox) == 1
+    assert len(app.mail.outbox) == 2
     assert "user" in requests[0]
     assert "login_token" in requests[0]
 
@@ -94,8 +94,8 @@ def test_passwordless_template(app, client, get_message):
     # in order to check all context vars since the default template
     # doesn't have all of them.
     with capture_passwordless_login_requests() as requests:
-        with app.mail.record_messages() as outbox:
-            client.post("/login", data=dict(email="joe@lp.com"), follow_redirects=True)
+        client.post("/login", data=dict(email="joe@lp.com"), follow_redirects=True)
+        outbox = app.mail.outbox
         assert len(outbox) == 1
         matcher = re.findall(r"\w+:.*", outbox[0].body, re.IGNORECASE)
         # should be 4 - link, email, token, config item
@@ -206,3 +206,11 @@ def test_spa_get_bad_token(app, client, get_message):
         msg = get_message("INVALID_LOGIN_TOKEN")
         assert msg == qparams["error"].encode("utf-8")
     assert len(flashes) == 0
+
+
+def test_deprecated(app, sqlalchemy_datastore):
+    with warnings.catch_warnings(record=True) as w:
+        warnings.simplefilter("always")
+        security = Security()
+        security.init_app(app, sqlalchemy_datastore)
+        assert any("passwordless feature" in str(m.message) for m in w)
