@@ -1,4 +1,4 @@
-# :copyright: (c) 2019-2021 by J. Christopher Wagner (jwag).
+# :copyright: (c) 2019-2022 by J. Christopher Wagner (jwag).
 # :license: MIT, see LICENSE for more details.
 
 """
@@ -22,27 +22,34 @@ data and a mail sender that flashes what mail would be sent!
 """
 
 import datetime
-import typing as t
 import os
+import typing as t
 
 from flask import Flask, flash, render_template_string, request, session
+from flask_sqlalchemy import SQLAlchemy
 
-from flask.json import JSONEncoder
 from flask_security import (
+    MailUtil,
     Security,
+    WebauthnUtil,
     auth_required,
     current_user,
     login_required,
-    SQLAlchemySessionUserDatastore,
+    SQLAlchemyUserDatastore,
 )
+from flask_security.models import fsqla_v3 as fsqla
 from flask_security.signals import (
     us_security_token_sent,
     tf_security_token_sent,
     reset_password_instructions_sent,
+    user_not_registered,
     user_registered,
 )
 from flask_security.utils import hash_password, uia_email_mapper, uia_phone_mapper
 
+if t.TYPE_CHECKING:  # pragma: no cover
+    from flask_security.datastore import User
+
 
 def _find_bool(v):
     if str(v).lower() in ["true"]:
@@ -52,17 +59,29 @@ def _find_bool(v):
     return v
 
 
-class FlashMail:
-    def __init__(self, app):
-        app.extensions["mail"] = self
+class FlashMailUtil(MailUtil):
+    def send_mail(
+        self,
+        template: str,
+        subject: str,
+        recipient: str,
+        sender: t.Union[str, tuple],
+        body: str,
+        html: t.Optional[str],
+        user: "User",
+        **kwargs: t.Any,
+    ) -> None:
+        flash(f"Email body: {body}")
 
-    def send(self, msg):
-        flash(msg.body)
+
+SET_LANG = False
 
 
 def create_app():
     # Use real templates - not test templates...
-    app = Flask("view_scaffold", template_folder="../")
+    app = Flask(
+        "view_scaffold", template_folder="../", static_folder="../flask_security/static"
+    )
     app.config["DEBUG"] = True
     # SECRET_KEY generated using: secrets.token_urlsafe()
     app.config["SECRET_KEY"] = "pf9Wkove4IKEAXvy-cQkeDPhv9Cb3Ag-wyJILbq_dFw"
@@ -70,22 +89,41 @@ def create_app():
     app.config["SECURITY_PASSWORD_SALT"] = "156043940537155509276282232127182067465"
 
     app.config["LOGIN_DISABLED"] = False
-    app.config["WTF_CSRF_ENABLED"] = False
+    app.config["WTF_CSRF_ENABLED"] = True
+    app.config["REMEMBER_COOKIE_SAMESITE"] = "strict"
+    app.config["SESSION_COOKIE_SAMESITE"] = "strict"
     app.config["SECURITY_USER_IDENTITY_ATTRIBUTES"] = [
         {"email": {"mapper": uia_email_mapper, "case_insensitive": True}},
         {"us_phone_number": {"mapper": uia_phone_mapper}},
     ]
     # app.config["SECURITY_US_ENABLED_METHODS"] = ["password"]
     # app.config["SECURITY_US_ENABLED_METHODS"] = ["authenticator", "password"]
-
     # app.config["SECURITY_US_SIGNIN_REPLACES_LOGIN"] = True
+    # app.config["SECURITY_WAN_ALLOW_USER_HINTS"] = False
 
     app.config["SECURITY_TOTP_SECRETS"] = {
         "1": "TjQ9Qa31VOrfEzuPy4VHQWPCTmRzCnFzMKLxXYiZu9B"
     }
-    app.config["SECURITY_FRESHNESS"] = datetime.timedelta(minutes=0.5)
+    app.config["SECURITY_FRESHNESS"] = datetime.timedelta(minutes=1)
     app.config["SECURITY_FRESHNESS_GRACE_PERIOD"] = datetime.timedelta(minutes=2)
     app.config["SECURITY_USERNAME_ENABLE"] = True
+    app.config["SECURITY_PASSWORD_REQUIRED"] = True
+    app.config["SECURITY_MULTI_FACTOR_RECOVERY_CODES"] = True
+    app.config["SECURITY_RETURN_GENERIC_RESPONSES"] = True
+
+    class TestWebauthnUtil(WebauthnUtil):
+        def generate_challenge(self, nbytes: t.Optional[int] = None) -> str:
+            # Use a constant Challenge so we can use this app to generate gold
+            # responses for use in unit testing. See test_webauthn.
+            # NEVER NEVER NEVER do this in production
+            return "smCCiy_k2CqQydSQ_kPEjV5a2d0ApfatcpQ1aXDmQPo"
+
+        def origin(self) -> str:
+            # Return the RP origin - normally this is just the URL of the application.
+            # To test with ngrok - we need the https address that the browser originally
+            # sent - it is sent as the ORIGIN header - not sure if this should be
+            # default or just for testing.
+            return request.origin if request.origin else request.host_url.rstrip("/")
 
     # Turn on all features (except passwordless since that removes normal login)
     for opt in [
@@ -97,6 +135,7 @@ def create_app():
         "confirmable",
         "two_factor",
         "unified_signin",
+        "webauthn",
     ]:
         app.config["SECURITY_" + opt.upper()] = True
 
@@ -105,15 +144,31 @@ def create_app():
         app.config.from_envvar("SETTINGS")
     # Allow any SECURITY_ config to be set in environment.
     for ev in os.environ:
-        if ev.startswith("SECURITY_"):
+        if ev.startswith("SECURITY_") or ev.startswith("SQLALCHEMY_"):
             app.config[ev] = _find_bool(os.environ.get(ev))
-    mail = FlashMail(app)
-    app.mail = mail
 
-    app.json_encoder = JSONEncoder
+    # Create database models and hook up.
+    app.config["SQLALCHEMY_TRACK_MODIFICATIONS"] = False
+    db = SQLAlchemy(app)
+    fsqla.FsModels.set_db_info(db)
+
+    class Role(db.Model, fsqla.FsRoleMixin):
+        pass
+
+    class User(db.Model, fsqla.FsUserMixin):
+        pass
+
+    class WebAuthn(db.Model, fsqla.FsWebAuthnMixin):
+        pass
+
     # Setup Flask-Security
-    user_datastore = SQLAlchemySessionUserDatastore(db_session, User, Role)
-    Security(app, user_datastore)
+    user_datastore = SQLAlchemyUserDatastore(db, User, Role, WebAuthn)
+    app.security = Security(
+        app,
+        user_datastore,
+        webauthn_util_cls=TestWebauthnUtil,
+        mail_util_cls=FlashMailUtil,
+    )
 
     try:
         import flask_babel
@@ -133,6 +188,10 @@ def create_app():
         def get_locale():
             # For a given session - set lang based on first request.
             # Honor explicit url request first
+            global SET_LANG
+            if not SET_LANG:
+                session.pop("lang", None)
+                SET_LANG = True
             if "lang" not in session:
                 locale = request.args.get("lang", None)
                 if not locale:
@@ -143,20 +202,6 @@ def create_app():
                     session["lang"] = locale
             return session.get("lang", None).replace("-", "_")
 
-    @app.before_first_request
-    def clear_lang():
-        session.pop("lang", None)
-
-    # Create a user to test with
-    @app.before_first_request
-    def create_user():
-        init_db()
-        db_session.commit()
-        test_acct = "test@test.com"
-        if not user_datastore.find_user(email=test_acct):
-            add_user(user_datastore, test_acct, "password", ["admin"])
-            print("Created User: {} with password {}".format(test_acct, "password"))
-
     @app.after_request
     def allow_absolute_redirect(r):
         # This is JUST to test odd possible redirects that look relative but are
@@ -169,6 +214,18 @@ def create_app():
     def on_user_registered(myapp, user, confirm_token, **extra):
         flash(f"To confirm {user.email} - go to /confirm/{confirm_token}")
 
+    @user_not_registered.connect_via(app)
+    def on_user_not_registered(myapp, **extra):
+        if extra.get("existing_email"):
+            flash(f"Tried to register existing email: {extra['user'].email}")
+        elif extra.get("existing_username"):
+            flash(
+                f"Tried to register email: {extra['form_data'].email.data}"
+                f" with username: {extra['form_data'].username.data}"
+            )
+        else:
+            flash("Not registered response - but ??")
+
     @reset_password_instructions_sent.connect_via(app)
     def on_reset(myapp, user, token, **extra):
         flash(f"Go to /reset/{token}")
@@ -194,9 +251,13 @@ def create_app():
     @login_required
     def home():
         return render_template_string(
-            "{% include 'security/_messages.html' %}"
-            "{{ _fsdomain('Welcome') }} {{email}} !",
+            """
+            {% include 'security/_messages.html' %}
+            {{ _fsdomain('Welcome') }} {{email}} !
+            {% include "security/_menu.html" %}
+            """,
             email=current_user.email,
+            security=app.security,
         )
 
     @app.route("/basicauth")
@@ -212,31 +273,6 @@ def create_app():
     return app
 
 
-"""
-
-Datastore
-
-"""
-
-from sqlalchemy import create_engine
-from sqlalchemy.orm import scoped_session, sessionmaker
-from sqlalchemy.ext.declarative import declarative_base
-
-engine = create_engine("sqlite:////tmp/view_scaffold.db?check_same_thread=False")
-db_session = scoped_session(
-    sessionmaker(autocommit=False, autoflush=False, bind=engine)
-)
-Base = declarative_base()
-Base.query = db_session.query_property()
-
-
-def init_db():
-    # import all modules here that might define models so that
-    # they will be registered properly on the metadata.  Otherwise
-    # you will have to import them first before calling init_db()
-    Base.metadata.create_all(bind=engine)
-
-
 def add_user(ds, email, password, roles):
     pw = hash_password(password)
     roles = [ds.find_or_create_role(rn) for rn in roles]
@@ -250,57 +286,14 @@ def add_user(ds, email, password, roles):
     ds.commit()
 
 
-"""
-
-Required Models
-
-"""
-
-from flask_security import UserMixin, RoleMixin
-from sqlalchemy.orm import relationship, backref
-from sqlalchemy import Boolean, DateTime, Column, Integer, String, Text, ForeignKey
-
-
-class RolesUsers(Base):  # type: ignore
-    __tablename__ = "roles_users"
-    id = Column(Integer(), primary_key=True)
-    user_id = Column("user_id", Integer(), ForeignKey("user.id"))
-    role_id = Column("role_id", Integer(), ForeignKey("role.id"))
-
-
-class Role(Base, RoleMixin):  # type: ignore
-    __tablename__ = "role"
-    id = Column(Integer(), primary_key=True)
-    name = Column(String(80), unique=True)
-    description = Column(String(255))
-
-
-class User(Base, UserMixin):  # type: ignore
-    __tablename__ = "user"
-    id = Column(Integer, primary_key=True)
-    fs_uniquifier = Column(String(64), unique=True, nullable=False)
-    email = Column(String(255), unique=True)
-    username = Column(String(255), unique=True, nullable=True)
-    password = Column(String(255), nullable=False)
-    last_login_at = Column(DateTime())
-    current_login_at = Column(DateTime())
-    last_login_ip = Column(String(100))
-    current_login_ip = Column(String(100))
-    login_count = Column(Integer)
-    active = Column(Boolean())
-    confirmed_at = Column(DateTime())
-
-    tf_phone_number = Column(String(128))
-    tf_primary_method = Column(String(64))
-    tf_totp_secret = Column(String(255))
-
-    us_totp_secrets = Column(Text(), nullable=True)
-    us_phone_number = Column(String(128), nullable=True)
-
-    roles: t.List = relationship(
-        "Role", secondary="roles_users", backref=backref("users", lazy="dynamic")
-    )
-
-
 if __name__ == "__main__":
-    create_app().run(port=5001)
+    myapp = create_app()
+
+    with myapp.app_context():
+        myapp.security.datastore.db.create_all()
+        test_acct = "test@test.com"
+        if not myapp.security.datastore.find_user(email=test_acct):
+            add_user(myapp.security.datastore, test_acct, "password", ["admin"])
+            print("Created User: {} with password {}".format(test_acct, "password"))
+
+    myapp.run(port=5001)
